@@ -2,14 +2,12 @@
 import re
 import datetime
 import logging
+import time
 
 # local imports
 from chainqueue.cache import CacheTx
 from chainqueue.entry import QueueEntry
-from chainqueue.error import (
-        NotLocalTxError,
-        BackendIntegrityError,
-        )
+from chainqueue.error import NotLocalTxError
 from chainqueue.enum import (
         StatusBits,
         all_errors,
@@ -31,7 +29,7 @@ all_local_errors = all_errors() - StatusBits.NETWORK_ERROR
 re_u = r'^[^_][_A-Z]+$'
 class Store:
 
-    def __init__(self, chain_spec, state_store, index_store, counter, cache=None):
+    def __init__(self, chain_spec, state_store, index_store, counter, cache=None, sync=True):
         self.chain_spec = chain_spec
         self.cache = cache
         self.state_store = state_store
@@ -49,19 +47,21 @@ class Store:
                 'unset',
                 'name',
                 'modified',
+                'purge',
                 ]:
             setattr(self, v, getattr(self.state_store, v))
 
+        if not sync:
+            return
+
         sync_err = None
-        for i in range(2):
-            try:
-                self.state_store.sync()
-            except Exception as e:
-                sync_err = e
-                continue
+        try:
+            self.state_store.sync()
+        except Exception as e:
+            sync_err = e
 
         if sync_err != None:
-            raise BackendIntegrityError(sync_err)
+            raise FileNotFoundError(sync_err)
 
 
     def put(self, v, cache_adapter=CacheTx):
@@ -80,26 +80,36 @@ class Store:
 
     def get(self, k):
         v = None
-        for i in range(2):
-            s = self.index_store.get(k)
-            try:
-                self.state_store.sync()
-                v = self.state_store.get(s)
-            except FileNotFoundError:
-                continue
-            break
+        s = self.index_store.get(k)
+        err = None
+        try:
+            v = self.state_store.get(s)
+        except FileNotFoundError as e:
+            err = e
         if v == None:
-            raise NotLocalTxError(k)
+            raise NotLocalTxError('could not find tx {}: {}'.format(k, err))
         return (s, v,)
 
 
-    def by_state(self, state=0, not_state=0, limit=4096, strict=False, threshold=None):
+    def by_state(self, state=0, not_state=0, include_pending=False, limit=4096, strict=False, threshold=None):
         hashes = []
         i = 0
+  
+        refs_state = []
+        if state > 0:
+            if self.state_store.is_pure(state):
+                refs_state = self.state_store.list(state)
+            elif strict:
+                refs_state = self.state_store.list(state)
+            else:
+                for v in self.state_store.elements(state, numeric=True):
+                    refs_state += self.state_store.list(v)
+                refs_state = list(set(refs_state))
+        if include_pending:
+            refs_state += self.state_store.list(0)
 
-        refs_state = self.state_store.list(state)
         refs_state.sort()
-        
+
         for ref in refs_state:
             v = from_key(ref)
             hsh = v[2]
@@ -110,10 +120,12 @@ class Store:
                 if item_state & state != item_state:
                     continue
 
-            logg.info('state {} {}'.format(ref, item_state))
             if item_state & not_state > 0:
                 continue
 
+            item_state_str = self.state_store.name(item_state)
+            logg.info('state {} {} ({})'.format(ref, item_state_str, item_state))
+
             if threshold != None:
                 v = self.state_store.modified(ref)
                 if v > threshold:
@@ -125,7 +137,7 @@ class Store:
             if limit > 0 and i == limit:
                 break
 
-        hashes.sort()
+        #hashes.sort()
         return hashes
 
 
@@ -149,7 +161,7 @@ class Store:
 
 
     def pending(self, limit=4096):
-        return self.by_state(state=0, limit=limit, strict=True)
+        return self.by_state(include_pending=True, limit=limit, strict=True)
 
 
     def reserve(self, k):
